#!/bin/sh

OVERLAY_STRING=""

check_overlay()
{
	echo check overlay: $OVERLAY_STRING
	cat /boot/config.txt | grep -ix $OVERLAY_STRING > /dev/null

	#if overlay not found
	if [ "$?" != "0" ]; then
		echo overlay not found

		mount -o rw,remount /boot
		./scripts/check_overlay.php $OVERLAY_STRING
		sync
		umount /boot
		mount /boot		

		reboot

	else
		echo overlay found
	fi
}

# check RPI model for different USB overlays
cat /sys/firmware/devicetree/base/model | grep -i "model a" > /dev/null
if [ "$?" = "0" ]; then
	OVERLAY_STRING="dtoverlay=dwc2,dr_mode=peripheral"
	check_overlay
fi

cat /sys/firmware/devicetree/base/model | grep -i "zero" > /dev/null
if [ "$?" = "0" ]; then
	OVERLAY_STRING="dtoverlay=dwc2"
	check_overlay
fi



# copy config from /boot with replace windows \r symbols
mount /boot > /dev/null 2>&1
cat /boot/cosmostreamer-pocket.conf | sed 's/\r$//' > /run/cosmostreamer.conf

. ./cosmostreamer.conf

echo powersave > /sys/devices/system/cpu/cpu0/cpufreq/scaling_governor

# run HDMI display
if [ "$hdmi_enabled" = "1" ] ; then
	echo using HDMI output
	#./scripts/hdmi-change.sh 1 > /dev/null 2>&1 &	
else
	echo using TV output
	./scripts/hdmi-change.sh 0 > /dev/null 2>&1 &
fi

# check update file in /boot
UPDATE_FILENAME=`ls /boot/*.tar.gz 2>&1`
if [ "$?" = "0" ] ; then
    ./update.sh $UPDATE_FILENAME
fi

# start wlans manager
./scripts/wlan-switch.php &

sudo /usr/bin/fbi -d /dev/fb0 -fitwidth -noverbose -T 2  ./images/background.jpg > /dev/null 2>&1

# stop intro
killall -q intro.sh intro

killall -q hud.sh usb-receiver queue display.sh display

#./scripts/display.sh > /dev/null 2>&1 &
./scripts/hud.sh > /dev/null 2>&1 &
./scripts/usb-receiver.sh > /dev/null 2>&1 &

# run RC receiver
./scripts/rc.sh > /dev/null 2>&1 &

# run internal camera
./scripts/camera.sh > /dev/null 2>&1 &

# run WebSockets data bridge
./scripts/ws-data.sh > /dev/null 2>&1 &

# run watchdog
./bin/watchdog -i 5 > /dev/null 2>&1 &

# run main application loop
while [ 1 ] ; do
    ./bin/cosmostreamer-pocket >> /dev/null 2>&1
    sleep 1
done
