const MOVEMODE_NONE = 0;
const MOVEMODE_ABS = 1;
const MOVEMODE_REL = 2;

var moveMode = MOVEMODE_REL;
var startX = 0, startY = 0;
var mouseButtonDown = false;

var dbltapTimestamp = 0;
var fsTimestamp = 0;

$(document).ready(function() {
	var win = $(this);
	ResizeInterface(win);

	SetCameraTime();

	$(window).on('resize', function(){
		var win = $(this);
		ResizeInterface(win);
	});

	$('#overlay').on("touchstart", onStartMove);
	$('#overlay').on("touchmove", onMove);
	$('#overlay').on("mousedown", onStartMove);
	$('#overlay').on("touchend", onEndMove);
	$('#overlay').on("mousemove", onMove);
	$('#overlay').on("mouseup", onEndMove);
	$('#overlay').on("mouseleave", onEndMove);

	$('#video_canvas').on("touchstart", onStartMove);
	$('#video_canvas').on("touchmove", onMove);
	$('#video_canvas').on("mousedown", onStartMove);
	$('#video_canvas').on("touchend", onEndMove);
	$('#video_canvas').on("mousemove", onMove);
	$('#video_canvas').on("mouseup", onEndMove);
	$('#video_canvas').on("mouseleave", onEndMove);

	function onMove(event) {
		if (!mouseButtonDown) return;

		var pageX = event.pageX == undefined ? event.originalEvent.touches[0].pageX : event.pageX;
		var pageY = event.pageY == undefined ? event.originalEvent.touches[0].pageY : event.pageY;

		var windowWidth = parseInt($('#overlay').css('width'));
		var windowHeight = parseInt($('#overlay').css('height'));
		var x = parseInt(pageX);
		var y = parseInt(pageY);

		switch (moveMode) {
			case MOVEMODE_ABS:
				x = parseInt(x / windowWidth * 6400 - 3200);
				y = parseInt(y / windowHeight * 1620 - 1300);
				send_command('position', 'move', x, y);
			break;

			case MOVEMODE_REL:
				x = parseInt((x - startX) * 4000 / 600);
				y = parseInt((y - startY) * 4000 / 600);
				send_command('position', 'moverel', x, y);
			break;
		}

		event.preventDefault();
	}

	function onStartMove(event) {
	    if ($("#central_panel").css("display") != "none") { $("#central_panel").css("display","none"); }
	    if ($("#shottype_panel").css("display") != "none") { $("#shottype_panel").css("display","none"); }
		
		CloseBoxSettings();

		mouseButtonDown = true;

		var pageX = event.pageX == undefined ? event.originalEvent.touches[0].pageX : event.pageX;
		var pageY = event.pageY == undefined ? event.originalEvent.touches[0].pageY : event.pageY;

		var windowWidth = parseInt($('#overlay').css('width'));
		var windowHeight = parseInt($('#overlay').css('height'));
		var x = parseInt(pageX);
		var y = parseInt(pageY);

		switch (moveMode) {
			case MOVEMODE_ABS:
				x = parseInt(x / windowWidth * 6400 - 3200);
				y = parseInt(y / windowHeight * 1620 - 1300);
				send_command('position', 'move', x, y);
			break;

			case MOVEMODE_REL:
				startX = x;
				startY = y;
			break;
		}

		event.preventDefault();
	}

	function onEndMove(event) {
		if (!mouseButtonDown) return;
		mouseButtonDown = false;
		send_command('position', 'moverel', 0, 0);
		startX = 0;
		startY = 0;
		event.preventDefault();

		// check double tap
		var timestamp = new Date().getTime();
		var delta = timestamp - dbltapTimestamp;
		dbltapTimestamp = timestamp;
		if (delta <= 200) {
			ToggleFullscreen();
			return;
		}
	}

	ShowHelloMessage();
});

function ToggleFullscreen() {
	var timestamp = new Date().getTime();
	var delta = timestamp - fsTimestamp;
	if (delta < 500) return;
	fsTimestamp = timestamp;

// if already full screen; exit else go fullscreen
if (
	document.fullscreenElement ||
	document.webkitFullscreenElement ||
	document.mozFullScreenElement ||
	document.msFullscreenElement
) {
	if (document.exitFullscreen) {
		document.exitFullscreen();
	} else if (document.mozCancelFullScreen) {
		document.mozCancelFullScreen();
	} else if (document.webkitExitFullscreen) {
		document.webkitExitFullscreen();
	} else if (document.msExitFullscreen) {
		document.msExitFullscreen();
	}
} else {
	element = $('body').get(0);
	if (element.requestFullscreen) {
		element.requestFullscreen();
	} else if (element.mozRequestFullScreen) {
		element.mozRequestFullScreen();
	} else if (element.webkitRequestFullscreen) {
		element.webkitRequestFullscreen(Element.ALLOW_KEYBOARD_INPUT);
	} else if (element.msRequestFullscreen) {
		element.msRequestFullscreen();
	}
}
ShowHelloMessage();
}

function isFullscreen() {
	if ( document.fullscreenElement || document.webkitFullscreenElement || document.mozFullScreenElement || document.msFullscreenElement ) return true; else return false;
}


function setlog(text) {
	$('#logwindow').html(text);
}

function ShowMainMessage(text) {
	if (text == undefined) {
		$('#mainmessage').html("");
		$('#mainmessage').css("display","none");
	} else {
		$('#mainmessage').html(text);
		$('#mainmessage').css("display","block");
	}
}

function ShowHelloMessage() {
	$("#hellomessage").css("display", "block");
	setTimeout(function(){$("#hellomessage").fadeOut(1000);}, 1000);
}

function makeTime(value) {
	var hours = parseInt(value / 3600);
	var minutes = parseInt((value % 3600) / 60);
	var seconds = parseInt(value - (hours * 3600) - minutes * 60);
	var hoursStr = (hours < 10) ? "0" + hours : "" + hours;
	var minutesStr = (minutes < 10) ? "0" + minutes : "" + minutes;
	var secondsStr = (seconds < 10) ? "0" + seconds : "" + seconds;
	return hoursStr + ":" + minutesStr + ":" + secondsStr;
}

function ResizeInterface(win) {
	//setlog(win.width() + "x" + win.height());
	var scale;
	var scale_x = win.width() / 1280;
	if (win.width() > win.height()) scale = win.width() / 1280; else scale = win.height() / 720;
	var newsize;

	newsize = parseInt(24.0 * scale_x);
	if (newsize > 16) newsize = 16;
	$(".scalable-text").css("font-size", newsize + "px");

	newsize = parseInt(60.0 * scale);
	$("#shot_buttons").css("width", newsize + "px").css("height", newsize + "px");
	$("#rec_buttons").css("width", newsize + "px").css("height", newsize + "px");	

	$("#image_settings").css("width", newsize + "px").css("height", newsize + "px");
	$("#cam_settings").css("width", newsize + "px").css("height", newsize + "px");

	$("#camera_mode_photo").css("width", newsize + "px").css("height", newsize + "px");
	$("#camera_mode_video").css("width", newsize + "px").css("height", newsize + "px");

	$("#zoom_in").css("width", newsize + "px").css("height", newsize + "px");
	$("#zoom_out").css("width", newsize + "px").css("height", newsize + "px");
	$("#opt_zoom_in").css("width", newsize + "px").css("height", newsize + "px");
	$("#opt_zoom_out").css("width", newsize + "px").css("height", newsize + "px");
}

function Vibrate(value) {
	try { navigator.vibrate(value); } catch (e) {}
}

function SetCameraTime() {
	var d = new Date();
	var time = 
		d.getFullYear() + "" + 
		('0'+(d.getMonth() + 1)).slice(-2) + "" + 
		('0'+d.getDate()).slice(-2) + "" + 
		('0'+d.getHours()).slice(-2) + "" + 
		('0'+d.getMinutes()).slice(-2) + "" + 
		('0'+d.getSeconds()).slice(-2);

	send_command("time", "", time, "");

	setTimeout(SetCameraTime, 5000);
	time = null;
	d = null;
}
