const HEADER_LEN = 12;

const TelemetryType = {
	TELEMETRY_POSITION : 0,
	TELEMETRY_POWER : 1,
	TELEMETRY_IMAGE : 2,
	TELEMETRY_ZOOM : 3,
	TELEMETRY_RCSTATE : 4,
	TELEMETRY_CONNECTIONSTATE : 5,
	TELEMETRY_DEVICESTATUS1 : 6,
	TELEMETRY_RECSTATUS : 7,
	TELEMETRY_ORIENTATION : 8,
	TELEMETRY_WLANSTATUS : 9,
	TELEMETRY_RAWPOSITION : 10,
	TELEMETRY_FWVERSION : 11,
	TELEMETRY_FOCUS : 12,
	TELEMETRY_FOCUS_POINT : 13,
	TELEMETRY_HISTOGRAM : 14,
	TELEMETRY_RCVALUES : 15,
	TELEMETRY_FOCUS_STATE : 16,
	TELEMETRY_JSVALUES : 17,
	TELEMETRY_LASTFILENAME : 18,
	TELEMETRY_POWER2 : 19,
	TELEMETRY_FOCUS2 : 20,
	TELEMETRY_TRACKING : 21
}

const OsmoConnectionType = {
	TELEMETRY_CONNECTIONSTATE_TELEMETRY : 0,
	TELEMETRY_CONNECTIONSTATE_VIDEO : 1
}

const ConnectionState = {
	TELEMETRY_CONNECTIONSTATE_DISCONNECTED : 0,
	TELEMETRY_CONNECTIONSTATE_CONNECTING : 1,
	TELEMETRY_CONNECTIONSTATE_CONNECTED : 2,
	TELEMETRY_CONNECTIONSTATE_DISABLED : 3
}


var data_connected = false;
var video_connected = false;

var recordActive = false;
var rec_busy = false;

var prev_shottype_group = -1;
var prev_shottype_item = 0;
var prev_shottype_interval = 0;
var prev_shottype_duration = 0;

var af_unlim = false;

function ParseMessage(buf) {
	var tmp; if (buf instanceof ArrayBuffer) tmp = new Uint8Array(buf); else return;
	var msg = tmp.subarray(HEADER_LEN);

	if (String.fromCharCode(msg[0]) == "{") {
		ParseSettings(msg);
		return;
	}

	var param1 = msg[1] + (msg[2] << 8);
	var param2 = msg[3] + (msg[4] << 8);
	var param3 = msg[5] + (msg[6] << 8);

	var param1b = msg[1];
	var param2b = msg[2];
	var param3b = msg[3];
	var param4b = msg[4];
	var param5b = msg[5];
	var param6b = msg[6];

	switch (msg[0]) {
		case TelemetryType.TELEMETRY_DEVICESTATUS1:
			var license_ok = (((param1b >> 0) & 1) == 1);
			var ap_state = (((param1b >> 1) & 1) == 1);
			var hdmi_state = (((param1b >> 2) & 1) == 1);
			var inet_available = (((param1b >> 3) & 1) == 1);
			var camera_active = (((param1b >> 4) & 1) == 1);
			var camera_disabled = (((param1b >> 5) & 1) == 1);
			var camera_sleeps = (((param1b >> 7) & 0x01) == 1);
			var view_clients_count = param2b;
		 	var custom_focus_assistant_enabled = (param4b & 0x01) == 1;
			var onboard_recording = ((param4b >> 1) & 0x01) == 1;

			$("#custom_fa_enabled").prop("checked", custom_focus_assistant_enabled ? "checked" : "");

			var str = "clients: " + view_clients_count;
			str += inet_available ? ", inet" : "";
			str += hdmi_state ? ", HDMI" : ", TV-out";
			str += ap_state ? ", AP" : "";
			$("#infostatus").html(str);

			$("#onboardrecstatus").html(onboard_recording ? "Onboard REC" : "");

			$("#badlicensemessage").css("display", license_ok ? "none" : "block");

			$("#cam_enable").html(camera_disabled ? "Enable Osmo connection" : "Disable Osmo connection");

			var protocols = "";
			if (((param3b >> 0) & 1) == 1) protocols += "UDP ";
			if (((param3b >> 1) & 1) == 1) protocols += "RTMP ";
			if (((param3b >> 2) & 1) == 1) protocols += "MPEG-TS ";
			if (((param3b >> 3) & 1) == 1) protocols += "TCP ";
			if (((param3b >> 4) & 1) == 1) protocols += "RTSP ";
			$("#protocolsstatus").html(protocols);

			ShowCameraSleeps(camera_sleeps);
		break;

		case TelemetryType.TELEMETRY_CONNECTIONSTATE:
			switch (param1b) {
				case OsmoConnectionType.TELEMETRY_CONNECTIONSTATE_TELEMETRY:
					switch (param2b) {
						case ConnectionState.TELEMETRY_CONNECTIONSTATE_CONNECTED: 
							data_connected = true;
							$("#overlay").css("display","block");
							$("#left_panel").css("display","block");
							$("#telemetrystate").html("");
						break;
						case ConnectionState.TELEMETRY_CONNECTIONSTATE_CONNECTING:
							data_connected = false;
							$("#overlay").css("display","none");
							$("#left_panel").css("display","none");
							$("#right_panel").css("display","none");
							$("#telemetrystate").html("telemetry connecting");
						break;
						case ConnectionState.TELEMETRY_CONNECTIONSTATE_DISABLED:
							data_connected = false;
							$("#overlay").css("display","none");
							$("#left_panel").css("display","none");
							$("#right_panel").css("display","none");
							$("#telemetrystate").html("telemetry disabled");
						break;
						case ConnectionState.TELEMETRY_CONNECTIONSTATE_DISCONNECTED:
							data_connected = false;
							$("#overlay").css("display","none");
							$("#left_panel").css("display","none");
							$("#right_panel").css("display","none");
							$("#telemetrystate").html("telemetry disconnected");
							$("#focus_block").css("display","none");
						break;
					}
				break;

				case OsmoConnectionType.TELEMETRY_CONNECTIONSTATE_VIDEO:
					switch (param2b) {
						case ConnectionState.TELEMETRY_CONNECTIONSTATE_CONNECTED:
							$("#videostate").html("");
							video_connected = true;
						break;
						case ConnectionState.TELEMETRY_CONNECTIONSTATE_CONNECTING:
							$("#videostate").html("video connecting");
							video_connected = false;
						break;
						case ConnectionState.TELEMETRY_CONNECTIONSTATE_DISABLED:
							$("#videostate").html("video disabled");
							video_connected = false;
						break;
						case ConnectionState.TELEMETRY_CONNECTIONSTATE_DISCONNECTED:
							$("#videostate").html("video disconnected");
							video_connected = false;
						break;
					}
				break;
			}
		break;

		case TelemetryType.TELEMETRY_FWVERSION:
			$("#field_fw_version").html(param1b + "." + param2b + "." + param3b);

			switch (param4b) {
				case 0:
					// Cosmostreamer for Osmo
				break;

				case 1:
					// Cosmostreamer Wired
				break;

				case 2:
					// Cosmostreamer for Pocket Osmo

					$("#opt_zoom_in").css('display','none');
					$("#opt_zoom_out").css('display','none');
					$("#zoom_in").css('display','none');
					$("#zoom_out").css('display','none');

					$("#realtime_af_block").css('display','none');
					$("#zoom_adjusted_smothtrack_block").css('display','none');
					$("#afa_block").css('display','none');
					$("#mfa_block").css('display','none');
					$("#cfa_block").css('display','none');

					$("#wifi_block_title").css('display','none');
					$("#wifi_block").css('display','none');

					$("#power_block_title").css('display','none');
					$("#power_block").css('display','none');
					$("#power_block2").css('display','none');

					$("#horizont_block_title").css('display','none');
					$("#horizont_block").css('display','none');

					$("#cam_enable").css('display','none');
				break;
			}
			
		break;

		case TelemetryType.TELEMETRY_WLANSTATUS:
			var quality = msg[1];
			var quality_max = msg[2];
			var signal_level = msg[3] > 127 ? msg[3] - 256 : msg[3];
			var freq = msg[4] + (msg[5] << 8);
			var ssid = "";
			for (var i = 6; i < msg.length; i++) {
				if (msg[i] == 0) break;
				ssid += String.fromCharCode(msg[i]);
			}

			var str = "\"" + ssid + "\"" + " quality " + quality + "/" + quality_max + " signal " + signal_level + " freq " + freq + " MHz";
			$("#wlanstatus").html(ssid == "" ? "Wi-Fi not connected" : str);
		break;

		case TelemetryType.TELEMETRY_POWER:
			$("#value_battery").css("color", param1 <= 15 ? "red" : "white");
			$("#value_battery").html((param2 / 1000.0).toFixed("2") + "V " + param1 + "%");
		break;

		case TelemetryType.TELEMETRY_POSITION:
			
			$("#value_pan").html((Int16(param1) / 100.0).toFixed("0"));
            $("#value_tilt").html((Int16(param2) / 100.0).toFixed("0"));
            $("#value_roll").html((Int16(param3) / 100.0).toFixed("0"));
		break;

		case TelemetryType.TELEMETRY_RECSTATUS:
			// Camera mode
            // 0 - photo
            // 1 - video
			var camera_mode = msg[1];

			// Recording status
			// bit 3 : 1 - shooting
			// bits 6 7 - 01 - record starting, 10 - record in progress, 11 - record stopping
			var rec_state = msg[2];

			// Photo status
			// bit 0 = 0 - makes shot
			var photo_state = 0;

			// Photo status 2
			// bit 3 = 1 - photo busy
			// bit 4 = 1 - timelapse or interval shoting in progress
			var photo_state2 = 0;

			try {
				photo_state = msg[21];
				photo_state2 = msg[22];
			} catch (e) { }

			// microSD free space in Megabytes
			var sd_free = (msg[3] + (msg[4] << 8) + (msg[5] << 16) + (msg[6] << 24));
			// microSD free space in JPEG photos
			var jpeg_left = (msg[7] + (msg[8] << 8) + (msg[9] << 16) + (msg[10] << 24));
			// estimated record time left
			var rec_time_left = (msg[11] + (msg[12] << 8) + (msg[13] << 16) + (msg[14] << 24));
			// current record time
			var rec_time = (msg[15] + (msg[16] << 8));

			// SSD free space in Megabytes
			var ssd_free = (msg[17] + (msg[18] * 256) + (msg[19] * 65536) + (msg[20] * (2^24)));

			var rec_busy = false;
			var shot_makes = false;
			var shot_busy = false;
			var pano_busy = false;
			var rec_state_message = "";

			if ((photo_state2 >> 4 & 1) == 1) {
				// pano or interval shot is active
				pano_busy = true;
				rec_state_message = "LOOP";
			}

			if ((photo_state2 >> 3 & 1) == 1) {
				shot_busy = true;
				rec_state_message = "BUSY";
			}

			if ((rec_state >> 3 & 1) == 1) {
				shot_makes = true;
				rec_state_message = "SHOT";
			}

			switch ((rec_state >> 6) & 0x03) {
				case 1: rec_state_message = "REC starting"; recordActive = false; rec_busy = true; break;
				case 2: rec_state_message = "REC"; recordActive = true; rec_busy = true; break;
				case 3: rec_state_message = "REC stopping"; recordActive = false; rec_busy = true; break;
			}

			$("#block_videoinfo").css("display", camera_mode == 1 ? "inline" : "none");
			$("#block_photoinfo").css("display", camera_mode == 0 ? "inline" : "none");

			$("#rec_buttons").css("display", camera_mode == 1 ? "block" : "none");
			$("#shot_buttons").css("display", camera_mode == 0 ? "block" : "none");

			if (camera_mode == 1) {
				$("#record_start_image").css("display", recordActive ? "none" : "block");
				$("#record_stop_image").css("display", recordActive ? "block" : "none");
			} else {
				$("#record_start_image").css("display", "none");
				$("#record_stop_image").css("display", "none");
			}

			if (camera_mode == 0) {
				if (pano_busy) {
					$("#shot_stop_image").css("display", "block");
					$("#shot_image").css("display", "none");
					$("#shot_busy_image").css("display", "none");
				} else {
					$("#shot_stop_image").css("display", "none");
					if (shot_busy) {
						$("#shot_image").css("display", "none");
						$("#shot_busy_image").css("display", "block");
					} else {
						$("#shot_busy_image").css("display", "none");
						$("#shot_image").css("display", shot_makes ? "none" : "block");
					}
				}				

				$("#shot_settings_block").css("display", "block");				
			} else {
				$("#shot_image").css("display", "none");
				$("#shottype_panel").css("display", "none");
				$("#shot_settings_block").css("display", "none");				
			}

			$("#value_cameramode").html(camera_mode == 1 ? "video" : "photo");

			$("#camera_mode_photo").css("opacity", camera_mode == 0 ? 1 : 0.3);
			$("#camera_mode_video").css("opacity", camera_mode == 1 ? 1 : 0.3);

			$("#value_photoleft").html(jpeg_left);

			$("#value_sdfree").html((sd_free / 1000.0).toFixed("1") + "G");
			$("#value_ssdfree").html((ssd_free / 1024.0).toFixed("1") + "G");
			$("#value_recstate").html(rec_state_message);

			if (sd_free == 0) {
				$("#nosd_message").css("display", "inline");
				$("#value_sdfree").css("display", "none");
				$("#title_sdfree").css("display", "none");
			} else {
				$("#nosd_message").css("display", "none");
				$("#value_sdfree").css("display", "inline");
				$("#title_sdfree").css("display", "inline");
			}

			if (ssd_free == 0) {
				$("#value_ssdfree").css("display", "none");
				$("#title_ssdfree").css("display", "none");
			} else {
				$("#value_ssdfree").css("display", "inline");
				$("#title_ssdfree").css("display", "inline");
			}

			$("#value_rectime").html(makeTime(rec_time) + " / " + makeTime(rec_time_left));
		break;

		case TelemetryType.TELEMETRY_IMAGE:
			// shot mode
			var shot_mode = msg[24];
			var shot_mode_str = "";
			switch (shot_mode) {
				case 0: shot_mode_str = "RAW"; break;
				case 1: shot_mode_str = "JPG"; break;
				case 2: shot_mode_str = "JPG+RAW"; break;
			}

			// AE (auto exposition)
			var ae = msg[15];
			var ae_str = ae == 1 ? "locked" : "unlocked";

			// Video mode
			// 1 - auto, 2 - shutter, 3 - aperture, 4 - manual
			var video_mode = msg[1];
			var video_mode_str = "";
			switch (video_mode) {
				case 1: video_mode_str = "auto"; break;
				case 2: video_mode_str = "shutter"; break;
				case 3: video_mode_str = "aperture"; break;
				case 4: video_mode_str = "manual"; break;
				default: video_mode_str = ""; break;
			}
			
			// Image aspect ratio
			// 0 - 4:3
			// 1 - 16:9
			var aspect_ratio = msg[2];
			var aspect_ratio_str = (aspect_ratio == 0) ? "4:3" : "16:9";

			// Image video standard
			// 0 - PAL
			// 1 - NTSC
			var video_std = msg[19];
			var video_std_str = (video_std == 0) ? "PAL" : "NTSC";

			// Anti=flicker
			// 0 - Auto
			// 1 - 50 Hz
			// 2 - 60 Hz
			var antiflicker = msg[20];
			var antiflicker_str = "auto";
			switch (antiflicker) {
				case 1: antiflicker_str = "50Hz"; break;
				case 2: antiflicker_str = "60Hz"; break;
			}

			// 0 - Standard
			// 1 - Scenery
			// 2 - Soft
			// 3 - Custom
			var scene = msg[3];
			var scene_str = "";
			switch (scene) {
				case 0: scene_str = "standard"; break;
				case 1: scene_str = "scenery"; break;
				case 2: scene_str = "soft"; break;
				case 3: scene_str = "custom"; break;
			}

			// Color mode
			var color_mode = msg[4];
			var color_mode_str = "";
			switch (color_mode) {
				case 23: color_mode_str = "d-log"; break;
				case 6: color_mode_str = "d-cinelike"; break;
				case 0: color_mode_str = "none"; break;
				case 1: color_mode_str = "art"; break;
				case 4: color_mode_str = "black white"; break;
				case 5: color_mode_str = "vivid"; break;
				case 14: color_mode_str = "beach"; break;
				case 16: color_mode_str = "dream"; break;
				case 18: color_mode_str = "classic"; break;
				case 19: color_mode_str = "nostalgia"; break;
				case 7: color_mode_str = "portrait"; break;
			}

			// EV mode ( multipled by 10)
			var ev_mode = Int8(msg[5]);
			var ev_mode_str = (ev_mode / 10.0).toFixed("1");
			if (ev_mode > 0) ev_mode_str = "+" + ev_mode_str;
			if (ev_mode == 0) ev_mode_str = "0";

			// Current WB
			var wb_mode = msg[6];
			var wb_mode_str = "";
			switch (wb_mode) {
				case 0: wb_mode_str = "auto"; break;
				case 1: wb_mode_str = "sunny"; break;
				case 2: wb_mode_str = "cloudy"; break;
				case 4: wb_mode_str = "incandescent"; break;
				case 5: wb_mode_str = "neon"; break;
				default: wb_mode_str = "custom"; break;
			}

			// Video resolution
			var video_resolution = msg[7];
			var video_resolution_str = "";
			var video_resolution_str2 = "";
			switch (video_resolution) {
				case 0x04: video_resolution_str = "1280x720"; video_resolution_str2 = "720"; break;
				case 0x0A: video_resolution_str = "1920x1080"; video_resolution_str2 = "1080"; break;
				case 0x10: video_resolution_str = "3840x2160"; video_resolution_str2 = "3.8K"; break;
				case 0x16: video_resolution_str = "4096x2160"; video_resolution_str2 = "4K"; break;
				case 0x18: video_resolution_str = "2704x1520"; video_resolution_str2 = "2.7K"; break;
			}

			// Video FPS
			var video_fps = msg[8];
			var video_fps_str = "";
			switch (video_fps) {
				case 1: video_fps_str = "24"; break;
				case 2: video_fps_str = "25"; break;
				case 3: video_fps_str = "30"; break;
				case 4: video_fps_str = "48"; break;
				case 5: video_fps_str = "50"; break;
				case 6: video_fps_str = "60"; break;
				case 7: video_fps_str = "120"; break;
			}

			// Custom WB in Kelvein divided by 100
			var wb_temperature = msg[9];

			// ISO real value
			var iso = msg[10] + (msg[11] << 8);

			// ISO setting value
			var iso_type = msg[16];
			var iso_type_str = "auto";
			switch (iso_type) {
				case 0: iso_type_str = "auto"; break;
				case 3: iso_type_str = "100"; break;
				case 4: iso_type_str = "200"; break;
				case 5: iso_type_str = "400"; break;
				case 6: iso_type_str = "800"; break;
				case 7: iso_type_str = "1600"; break;
				case 8: iso_type_str = "3200"; break;
				case 9: iso_type_str = "6400"; break;
			}

			// some fun with shutter config value, thanx DJI :-(
			var shutter_config_sign = "";
			var shutter_config_str = "";
			var shutter_config = 0;
			var shutter_config_float = msg[14];
			if (msg[13] == 0) {
				shutter_config_sign = "\"";
				shutter_config = msg[12];
				shutter_config_str = shutter_config + ((shutter_config_float > 0) ? "." + shutter_config_float : "") + shutter_config_sign;
			} else if (msg[13] >= 128) {
				shutter_config = (msg[12] + ((msg[13] - 128) << 8));
				shutter_config_str = shutter_config + ((shutter_config_float > 0) ? "." + shutter_config_float : "") + shutter_config_sign;
			}
			if (shutter_config == 0) shutter_config_str = "";

			// some fun with shutter real current value, thanx DJI :-(
			var shutter_sign = "";
			var shutter_str = "";
			var shutter = 0;
			var shutter_float = msg[23];
			if (msg[22] == 0) {
				shutter_sign = "\"";
				shutter = msg[21];
				shutter_str = shutter + ((shutter_float > 0) ? "." + shutter_float : "") + shutter_sign;
			} else if (msg[22] >= 128) {
				shutter = (msg[21] + ((msg[22] - 128) << 8));
				shutter_str = shutter + ((shutter_float > 0) ? "." + shutter_float : "") + shutter_sign;
				shutter_str = "1/" + shutter_str;
			}
            if (shutter == 0) shutter_str = "";

			// aperture
			var aperture = (msg[17] + (msg[18] << 8)) / 100.0;
			var aperture_str = "";
			if (aperture == Math.floor(aperture)) {
				aperture_str = aperture.toFixed("0");
			} else {
				aperture_str = aperture.toFixed("1");
			}


			// photo shot type
			var shottype_group = 0;
			var shottype_item = 0;
			var shottype_interval = 0;
			var shottype_duration = 0;

			try {
				shottype_group = msg[25];
				shottype_item = msg[26];
				shottype_interval = msg[27] + (msg[28] << 8);
				shottype_duration = msg[29] + (msg[30] << 8);
			} catch (e) { }

			if (shottype_group != prev_shottype_group || shottype_item != prev_shottype_item || shottype_interval != prev_shottype_interval || shottype_duration != prev_shottype_duration) {
				prev_shottype_group = shottype_group;
				prev_shottype_item = shottype_item;
				prev_shottype_interval = shottype_interval;
				prev_shottype_duration = shottype_duration;

				var group_name = "";
				var item_name = "";
				var item_text = "";

				switch (shottype_group) {
					case 0:
						group_name = "single";
						if (shottype_item == 0) { item_name = "shot"; item_text = "shot"; }
						if (shottype_item == 1) { item_name = "10s"; item_text = "10 s"; }
						if (shottype_item == 2) { item_name = "hdr"; item_text = "HDR"; }
					break;
					case 1:
						group_name = "multiple";
						if (shottype_item == 0) { item_name = "3shots"; item_text = "3 shots"; }
						if (shottype_item == 1) { item_name = "5shots"; item_text = "5 shots"; }
						if (shottype_item == 2) { item_name = "7shots"; item_text = "7 shots"; }
						if (shottype_item == 3) { item_name = "aeb3"; item_text = "3 AEB"; }
						if (shottype_item == 4) { item_name = "aeb5"; item_text = "5 AEB"; }
					break;
					case 2:
						group_name = "interval";
						if (shottype_item == 0) { item_name = "10s"; item_text = "10 s"; }
						if (shottype_item == 1) { item_name = "30s"; item_text = "30 s"; }
					break;
					case 3:
						group_name = "pano";
						if (shottype_item == 0) { item_name = "180"; item_text = "180"; }
						if (shottype_item == 1) { item_name = "360"; item_text = "360"; }
						if (shottype_item == 2) { item_name = "180self"; item_text = "180 selfie"; }
						if (shottype_item == 3) { item_name = "manual"; item_text = "manual"; }
						if (shottype_item == 4) { item_name = "grid"; item_text = "grid"; }
					break;
					case 4:
						group_name = "timelapse";
						item_name = "timelapse";
						item_text = shottype_interval + "s / ";
						if (shottype_duration == 0) item_text += "unlim";
						if ((shottype_duration > 0) && (shottype_duration < 3600)) item_text += (shottype_duration / 60) + "min";
						if (shottype_duration >= 3600) item_text += (shottype_duration / 3600) + "hr";

						$("#param_timelapse_interval").val(shottype_interval + "s");

						var duration_option = "";
						if (shottype_duration == 0 ) duration_option = "unlim";
						if ((shottype_duration > 0) && (shottype_duration < 3600)) duration_option = (shottype_duration / 60) + "min";
						if (shottype_duration >= 3600) duration_option = (shottype_duration / 3600) + "hr";
						$("#param_timelapse_duration").val(duration_option);
					break;
				}

				$("#value_phototype").html(group_name + " " + item_text);
				$("#shot_settings").html(item_name);

				$('.shot_type_group_div').css('color','white');
				$('.shot_type_element').css('color','white');
				
				try { $('div[group="' + group_name + '"][item="' + item_name + '"]').css('color','yellow'); } catch (e) {}
				try { $('.shot_type_group_div[item="' + group_name + '"]').css('color','yellow'); } catch (e) {}
			}

			// end photo shot type


			$("#value_resolution").html(video_resolution_str2);
			$("#value_fps").html(video_fps_str);
			$("#value_iso").html(iso);
			$("#value_shutter").html(shutter_str);
			$("#value_aperture").html(aperture_str);
			$("#value_ev").html(ev_mode_str);
			$("#value_wb").html((wb_temperature * 100) + "K");
			$("#value_wbmode").html(wb_mode_str);
			$("#value_ae").css("color", ae == 1 ? "yellow" : "gray" );
			$("#value_color").html(color_mode_str);
			$("#value_vmode").html(video_mode_str);
			$("#value_photomode").html(shot_mode_str);

			$("#ae_locked").css("display", ae == 1 ? "block" : "none");
			$("#ae_unlocked").css("display", ae == 0 ? "block" : "none");

			$("#param_color").val(color_mode_str);
			$("#param_scene").val(scene_str);
			$("#param_wb").val(wb_mode_str);
			$("#param_videomode").val(video_mode_str);
			$("#param_shutterspeed").val(shutter_config_str.replace('"', 's'));
			$("#param_aperture").val(aperture_str);
			$("#param_iso").val(iso_type_str);
			$("#param_ev").val(ev_mode_str);
			//$("#param_scenemode").val();

			$("#param_imageaspect").val(aspect_ratio_str);
			$("#param_videostandard").val(video_std_str);
			$("#param_antiflicker").val(antiflicker_str);
			$("#param_shotmode").val(shot_mode_str);
			$("#param_videores").val(video_resolution_str + "@" + video_fps_str);


		break;

		case TelemetryType.TELEMETRY_ORIENTATION:
			$("#value_orientationlocked").css("color", msg[1] == 1 ? "yellow" : "gray");
			$("#param_orientation_lock").css("display", msg[1] == 1 ? "none" : "inline");
			$("#param_orientation_unlock").css("display", msg[1] == 1 ? "inline" : "none");
		break;

		case TelemetryType.TELEMETRY_RCVALUES:
			ShowRcState(msg[1]);

			if ($("#config_window").css("display") == "none") break;
			for (var i = 0; i < msg[2]; i++) {
				var value = msg[3 + i * 2] + (msg[3 + i * 2 + 1] << 8);
				ShowRcChannelValue(i, value);
			}
		break;

		case TelemetryType.TELEMETRY_FOCUS_POINT:
			var state = msg[5] + (msg[6] << 8);
            var mf_ass = state & 0x1;
            var af_ass = (state >> 1) & 0x1;

			$("#mfa_enabled").prop("checked", mf_ass ? "checked" : "");
			$("#afa_enabled").prop("checked", af_ass ? "checked" : "");
		break;

		case TelemetryType.TELEMETRY_FOCUS:
			var focus_ring_type = param1 & 0x1; //  ,    
			var focus_type = (param1 >> 1) & 0x1; //  ,  
			var focus_percentage = param2;
			var focus_distance = param3 / 100.0;

            $("#focus_block").css("display","inline");

            ShowFocusPercentage(focus_percentage);
            ShowFocusDistance(focus_distance);
			ShowFocusType(focus_ring_type, focus_type, af_unlim);
		break;

		case TelemetryType.TELEMETRY_FOCUS_STATE:
			af_unlim = param2b == 1;
		break;

		case TelemetryType.TELEMETRY_POWER2:
			if (param1 <= 15) {
				$("#value_battery").css("color", "red");
			} else {
				$("#value_battery").css("color", param2 == 1 ? "lightgreen" : "white");
			}
			$("#value_battery").html(param1 + "%");
		break;
	}

	buf = null;
	msg = null;
}
